//
//  ShapeyrOutlineViewController.m
//  Shapeyr
//
//  Created by Taylor Marks on 12/2/12.
//  Copyright (c) 2012 Taylor Marks. All rights reserved.
//

#import "ShapeyrOutlineViewController.h"

@implementation ShapeyrOutlineViewController
- (void)awakeFromNib {
    dragType = [NSArray arrayWithObject:@"factorialDragType"];
    [outlineView registerForDraggedTypes:dragType];
    NSSortDescriptor* sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"index" ascending:YES];
    [treeController setSortDescriptors:[NSArray arrayWithObject:sortDescriptor]];
}

- (BOOL)outlineView:(NSOutlineView*)outlineView writeItems:(NSArray*)items toPasteboard:(NSPasteboard *)pasteboard {
    [pasteboard declareTypes:dragType owner:self];
    draggedNode = [items objectAtIndex:0];
    draggedObject = [draggedNode representedObject];
    return YES;
}

- (BOOL)outlineView:(NSOutlineView *)mOutlineView acceptDrop:(id<NSDraggingInfo>)info item:(id)parent childIndex:(NSInteger)index {
    [draggedObject setValue:[parent representedObject] forKey:@"parent"];
    NSInteger dropIndex = (index == NSOutlineViewDropOnItemIndex) ? 0 : index;
    [draggedObject setValue:[NSNumber numberWithLong:dropIndex] forKey:@"index"];
    NSIndexPath* dropPath = (parent) ? [[parent indexPath] indexPathByAddingIndex:dropIndex] : [NSIndexPath indexPathWithIndex:dropIndex];
    [NSTimer scheduledTimerWithTimeInterval:0.02 target:self selector:@selector(moveToIndexPath:) userInfo:dropPath repeats:NO];
    return YES;
}

- (void)moveToIndexPath:(NSTimer*)timer {
    [treeController moveNode:draggedNode toIndexPath:[timer userInfo]];
}

- (BOOL)object:(NSManagedObject*)object isChildOf:(NSManagedObject*)possibleParent {
    // TODO: This can probably be simplified...
    if (object == possibleParent) {
        return YES;
    }
    
    possibleParent = [possibleParent valueForKey:@"parent"];
    while (possibleParent != NULL) {
        if (possibleParent == object) {
            return YES;
        }
        possibleParent = [possibleParent valueForKey:@"parent"];
    }
    return NO;
}

- (NSDragOperation)outlineView:(NSOutlineView*)outlineView validateDrop:(id <NSDraggingInfo>)info proposedItem:(id)proposedParent proposedChildIndex:(NSInteger)index {
    NSManagedObject* proposedParentObject = [proposedParent representedObject];
    NSManagedObject* currentParentObject = [draggedObject valueForKey:@"parent"];
    if (proposedParentObject == NULL) {
        // It's being proposed that this object gets moved to the root level.
        if (currentParentObject == NULL) {
            // This object is already at the root level.
            switch (index-[[draggedNode indexPath] indexAtPosition:([[draggedNode indexPath] length]-1)]) {
                case 0: case 1:
                    return NO; // This object wouldn't be getting moved anywhere in these cases. Don't propose it.
                default: break; // NSDragOperationGeneric will be returned just outside the if.
            }
        }
        return NSDragOperationGeneric;
    }
    
    if ([self object:draggedObject isChildOf:proposedParentObject]) {
        // One object cannot become its own child.
        return NO;
    } else if ([proposedParentObject isEqual:currentParentObject]) {
        // The objects share the same parent.
        switch (index-[[draggedNode indexPath] indexAtPosition:([[draggedNode indexPath] length]-1)]) {
            case 0: case 1:
                return NO; // This object wouldn't be getting moved anywhere in these cases. Don't propose it.
            default: break; // NSDragOperationGeneric will be returned just outside the if.
        }
    }
    
    return NSDragOperationGeneric;
}

#pragma mark Not necessary on account of Cocoa Bindings:

- (id)outlineView:(NSOutlineView*)outlineView objectValueForTableColumn:(NSTableColumn*)tableColumn byItem:(id)item {
    return NULL;
}

-(BOOL)outlineView:(NSOutlineView*)outlineView isItemExpandable:(id)item {
    return YES;
}

- (id)outlineView:(NSOutlineView *)outlineView child:(NSInteger)index ofItem:(id)item {
    return NULL;
}
@end
